#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtWidgets import *
from panebase import PaneBase


class NQU(QGroupBox):

    def __init__(self, title, presets):
        super().__init__()
        
        layout = QHBoxLayout()        

        self.name = QComboBox(None)
        self.name.setEditable(True)
        self.name.addItems(presets['names'])
        layout.addWidget(self.name)
        
        self.quantity = QDoubleSpinBox(None)
        self.quantity.setMaximum(999)
        layout.addWidget(self.quantity)
        
        self.unit = QComboBox(None)
        self.unit.setEditable(True)
        self.unit.addItems(presets['units'])
        layout.addWidget(self.unit)
        
        buttonBox = QDialogButtonBox(QDialogButtonBox.No)
        self.remove = buttonBox.button(QDialogButtonBox.No)
        self.remove.setText('Remove')
        layout.addWidget(buttonBox)
        
        self.setLayout(layout)
        self.setTitle(title)
        self.isFlat = True
        
    def value(self):
        return {self.title(): [self.name.currentText(), self.quantity.value(), self.unit.currentText()]}
      
    def load(self, data):
        key, value = list(data.items())[0]
        self.setTitle(key)
        self.name.setCurrentIndex(self.name.findText(value[0]))
        self.quantity.setValue(value[1])
        self.unit.setCurrentIndex(self.unit.findText(value[2]))

 
class BufferDialog(PaneBase):
 
    fields = {
        'ph': (QDoubleSpinBox, 'Ph value', '', 7.),

    }
    def __init__(self, parent, data={}, presets={}):
        if not presets:
            presets = {'names': [],
                       'units': ["mM", "µl", "ml", "cl", "dl", "L"]
                      }
        
        # must exist before loading data (in super's init)
        self.addButton = QPushButton("&Add Component")
        
        PaneBase.__init__(self, parent, "Buffer Data", data, presets)
        
        e = self.addfieldbydict('ph')
        e.setRange(0.0, 14.0)
        e.setSingleStep(0.5)
        
        from helpers import VariableWidget
        self.NQUs = VariableWidget(NQU, data.get('components', []), presets)
        self.elements.append((self.NQUs, 'components'))
        self.layout().addWidget(self.NQUs)
        
        self.addbottom()

        self.buttonBox.addButton(self.addButton, QDialogButtonBox.HelpRole)
        self.addButton.clicked.connect(self.NQUs.add)
        
    def slotRecord(self):
        for e in self.NQUs.components:
            if e.name.currentText() not in self.presets['names']:
                self.presets['names'].append(e.name.currentText())
            if e.unit.currentText() not in self.presets['units']:
                self.presets['units'].append(e.unit.currentText())
        super().slotRecord()

    def slotValid(self, valid):
        super().slotValid(valid)
        self.addButton.setEnabled(not valid)
