#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from panebase import PaneBase


class CryoDialog(PaneBase):
    
    fields = {
        'date': (QDateEdit, 'Date', '', QDate.currentDate().toString("yyyy-MM-dd")),
        'charging': (QSpinBox, 'Charging', 'mA', 2),
        'duration': (QSpinBox, 'Duration', 's', 10),
        'pressure': (QSpinBox, 'Pressure', 'mBar', 200),
        'tafter': (QSpinBox, 'Time after', 'min', 20),
        'temperature': (QDoubleSpinBox, 'Temperature', '°C', 10.),
        'humidity': (QDoubleSpinBox, 'Humidity', '%', 95.),
        'time': (QDoubleSpinBox, 'Blot Time', 's', 1),
        'force': (QSpinBox, 'Blot Force', '', 7),
        'quantity': (QDoubleSpinBox, 'Drop size', 'µl', 2),
    }
 
    def __init__(self, parent, data={}):

        PaneBase.__init__(self, parent, "Cryogenization Data", data)

        e = self.addfieldbydict('date')
        
        e = QLabel('Glow discharge')
        self.layout().addWidget(e)
        
        e = self.addfieldbydict('charging')
        e.setRange(0, 10000)
        
        e = self.addfieldbydict('duration')
        e.setRange(0, 100)

        e = self.addfieldbydict('pressure')
        e.setRange(0, 1000)

        e = self.addfieldbydict('tafter')
        e.setRange(0, 200)
        
        e = QLabel('Vitrification')
        self.layout().addWidget(e)

        e = self.addfieldbydict('temperature')
        e.setRange(0.0, 50.0)
        
        e = self.addfieldbydict('humidity')
        e.setRange(0.0, 100.0)
        
        e = self.addfieldbydict('time')
        e.setRange(0.0, 100.0)
        e.setSingleStep(0.1)
        
        e = self.addfieldbydict('force')
        e.setRange(0, 20)
        
        e = self.addfieldbydict('quantity')
        e.setRange(0, 15)
        e.setSingleStep(0.5)

        self.addbottom()
