#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import tarfile

class DataOnFS(QMessageBox):
 
    def __init__(self, parent, tarfilename):
        QDialog.__init__(self, parent)
        
        self.tarfilename = tarfilename
        
        now = QDateTime.currentDateTime()
        self.tmpdir = QDir.tempPath() + "/gridfiles-" + now.toString("yyyyMMddhhmmss") + "/"

        self.setWindowTitle("Data On FS")
        self.setText("Data has been unpacked to {0}.\nIt will be repacked when you close this.".format(QDir.toNativeSeparators(self.tmpdir)))
        
        self.finished.connect(self.slotRepack)
        
        QDir().mkpath(self.tmpdir)
        
        with tarfile.open(self.tarfilename, 'r') as tar:
            tar.extractall(self.tmpdir)


    def slotRepack(self):
        # make a backup
        now = QDateTime.currentDateTime()
        backup = self.tarfilename + "-backup-" + now.toString("yyyyMMddhhmmss")
        QFile.rename(self.tarfilename, backup)
        # rewrite all
        with tarfile.open(self.tarfilename, 'w') as tar:
            tar.add(self.tmpdir, arcname=".")
        QDir(self.tmpdir).removeRecursively()
