#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
import urllib.request
from base64 import b64encode
import os


def handlecreds(req):
    if '@' in req.host:
        pos = req.host.index('@')
        creds = req.host[:pos]
        b64 = b64encode(bytes(creds, 'utf8')).decode("ascii")
        req.add_header('Authorization', 'Basic ' + b64)
        req.host = req.host[pos+1:]

def push(fname, dest, path=None, buf=None):

    # if we got a buffer, copy from it instead of a file on storage
    if buf:
        data = buf
        buf.seek(0, os.SEEK_END)
        size = buf.tell()
        buf.seek(0)
    else:
        data = open(os.path.normcase(fname), 'rb')
        size = os.stat(fname).st_size
    
    # if directory given, try to create it
    if not path:
        path = '.'
    else:
        try:
            req = urllib.request.Request('/'.join(s.strip('/') for s in [dest, path]), method='MKCOL')
            handlecreds(req)
            urllib.request.urlopen(req)
        except:
            pass

    req = urllib.request.Request('/'.join(s.strip('/') for s in [dest, path, os.path.basename(fname)]), method='PUT', data=data)
    handlecreds(req)
    req.add_header('Content-length', size)

    urllib.request.urlopen(req)


def pull(fname, src, path='.', buf=None):
    import shutil

    req = urllib.request.Request('/'.join(s.strip('/') for s in [src, path, os.path.basename(fname)]))
    handlecreds(req)
    
    # if we got a buffer, copy into it instead of a file on storage
    out_file = buf if buf else open((fname), 'wb')
        
    with urllib.request.urlopen(req) as response:
        shutil.copyfileobj(response, out_file, length=16*1024*1024)

    if buf is None:
        out_file.close()
    else:
        buf.seek(0)

def move(fname, dest, newname, path=None):

    req = urllib.request.Request('/'.join(s.strip('/') for s in [dest, path, os.path.basename(fname)]), method='MOVE')
    handlecreds(req)
    req.add_header('Destination', '/'.join(s.strip('/') for s in [dest, path, newname]))

    urllib.request.urlopen(req)
