#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *


class GridImage(QGroupBox):
    def __init__(self, parent):
        super().__init__(parent)
        
        self.imagetype = 'misc'
        
        layout = QVBoxLayout()

        upload = QPushButton("Upload Image")
        upload.clicked.connect(self.slotUpload)
        layout.addWidget(upload)
        self.image = QLabel()
        self.image.mouseDoubleClickEvent = self.bigview
        layout.addWidget(self.image)
        self.hint = QLabel('Hint: double-click image to navigate through it!')
        self.hint.setVisible(False)
        layout.addWidget(self.hint)
        
        self.setLayout(layout)
        self.setTitle("")
        self.isFlat = True
        
    def value(self):
        return self.fname
    
    def show(self, data):
        pmap = QPixmap()
        pmap.loadFromData(data.read())
        self.image.setPixmap(pmap.scaled(300,300,Qt.KeepAspectRatio))
        self.pmap = pmap
        self.hint.setVisible(True)
      
    def load(self, data, location, imagesdir, gridname):
        from helpers import getimage
        self.fname = data
        self.location = location
        self.imagesdir = imagesdir
        self.gridname = gridname
        if self.fname:
            try:
                idata = getimage(location, imagesdir, self.fname)
                self.show(idata)
            except Exception as e:
                from helpers import alert
                alert('Could not retreive image: ' + str(e))

    def slotUpload(self):
        import os
        from helpers import addimage
        filename = QFileDialog.getOpenFileName(self, "Select Image")[0]
        if filename:
            try:
                split = filename.rsplit('.', 2)
                ext = split[1] if len(split) > 1 else ''
                fi = QFileInfo(self.gridname)
                base = fi.baseName()
                self.fname = base + '-' + self.imagetype + '.' + ext
                data = addimage(filename, self.location, self.imagesdir, self.fname)
                self.show(data)
            except Exception as e:
                from helpers import alert
                alert('Could not add image: ' + str(e))
                

    def zoom(self, event):
        
        factor = 1.1
        
        if event.angleDelta().y() < 0:
            factor = 1. / factor
        self.view.scale(factor, factor)
    
    def bigview(self, event):
        scene = QGraphicsScene()
        scene.addPixmap(self.pmap)
        self.view = QGraphicsView(scene)
        self.view.wheelEvent = self.zoom
        self.view.setDragMode(QGraphicsView.ScrollHandDrag)
        self.view.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
        self.view.show()
