#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016-2017
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtWidgets import *
from panebase import PaneBase


class UL(QGroupBox):

    def __init__(self, title, presets):
        super().__init__()

        layout = QHBoxLayout()

        self.user = QComboBox(None)
        self.user.setEditable(True)
        self.user.addItems(presets['users'])
        layout.addWidget(self.user)
        
        self.lab = QComboBox(None)
        self.lab.setEditable(True)
        self.lab.addItems(presets['labs'])
        layout.addWidget(self.lab)
        
        buttonBox = QDialogButtonBox(QDialogButtonBox.No)
        self.remove = buttonBox.button(QDialogButtonBox.No)
        self.remove.setText('Remove')
        layout.addWidget(buttonBox)
        
        self.setLayout(layout)
        self.setTitle(title)
        self.isFlat = True
        
    def value(self):
        return {self.title(): [self.user.currentText(), self.lab.currentText()]}
      
    def load(self, data):
        key, value = list(data.items())[0]
        self.setTitle(key)
        self.user.setCurrentIndex(self.user.findText(value[0]))
        self.lab.setCurrentIndex(self.lab.findText(value[1]))


class InfosDialog(PaneBase):
 
    fields = {
        'comment': (QTextEdit, 'Free Comments', '', '')
    }

    def __init__(self, parent, data={}, presets={}):
        if not presets:
            presets = {'users': [],
                       'labs': []
                      }

        # must exist before loading data (in super's init)
        self.addButton = QPushButton("&Add User")

        PaneBase.__init__(self, parent, "Infos", data, presets)

        from helpers import VariableWidget
        self.ULs = VariableWidget(UL, data.get('users', []), presets)
        self.elements.append((self.ULs, 'users'))
        self.layout().addWidget(self.ULs)

        self.addfieldbydict('comment')
        
        self.addbottom()

        self.buttonBox.addButton(self.addButton, QDialogButtonBox.HelpRole)
        self.addButton.clicked.connect(self.ULs.add)

    def slotRecord(self):
        for e in self.ULs.components:
            if e.user.currentText() not in self.presets['users']:
                self.presets['users'].append(e.user.currentText())
            if e.lab.currentText() not in self.presets['labs']:
                self.presets['labs'].append(e.lab.currentText())
        super().slotRecord()

    def slotValid(self, valid):
        super().slotValid(valid)
        self.addButton.setEnabled(not valid)
