#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from panebase import PaneBase
from helpers import RadioWidget

 
class MicroscopeDialog(PaneBase):
 
    fields = {
        'date': (QDateEdit, 'Date', '', QDate.currentDate().toString("yyyy-MM-dd")),
        'grid': (QComboBox, 'Grid Type', '', ''),
        'microscope': (QComboBox, 'Microscope', '', ''),
        'voltage': (QSpinBox, 'Voltage', 'kV', 100),
        'camera': (QComboBox, 'Camera Name', '', ''),
        'magnification': (QSpinBox, 'Magnification', 'x', 96000),
        'mindefocus': (QDoubleSpinBox, 'Minimum Defocus', 'µm', 0),
        'maxdefocus': (QDoubleSpinBox, 'Maximum Defocus', 'µm', 0),
        'state': (RadioWidget, 'Acquisition state', '', ''),

    }
    def __init__(self, parent, data={}, presets={}):
        if not presets:
            presets = {'microscope': [],
                       'camera': [],
                       'grid': []
                      }

        PaneBase.__init__(self, parent, "Microscope Data", data, presets)

        e = self.addfieldbydict('date')
        e = self.addfieldbydict('grid')
        e = self.addfieldbydict('microscope')
        e = self.addfieldbydict('voltage')
        e.setRange(100, 300)
        e.setSingleStep(50)
        e = self.addfieldbydict('camera')
        e = self.addfieldbydict('magnification')
        e.setRange(0, 200000)
        e.setSingleStep(1000)
        e = self.addfieldbydict('mindefocus')
        e.setRange(-12, 0)
        e.setSingleStep(0.1)
        e = self.addfieldbydict('maxdefocus')
        e.setRange(-12, 0)
        e.setSingleStep(0.1)
        self.radios = self.addfieldbydict('state')
        self.radios.setbuttons(__class__.getbuttons())
        
        self.addbottom()
    
    @staticmethod
    def getbuttons():
        buttons = [QRadioButton("Acquired"), QRadioButton("Not acquired"),
                   QRadioButton("Problem while acquiring"), QRadioButton("No image")]
        buttons[0].setStyleSheet("background-color: lightgreen")
        buttons[1].setStyleSheet("background-color: orange")
        buttons[2].setStyleSheet("background-color: yellow")
        buttons[3].setStyleSheet("background-color: lightskyblue")
        return buttons
