#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

 
class PaneBase(QDialog):
 
    def __init__(self, parent, title, data={}, presets={}):
        from helpers import myfield
        QDialog.__init__(self, parent)
        self.setWindowTitle(title)
        
        self.data = data
        self.presets = presets
        
        # list of variable elements to be greyed out by validation
        self.elements = []

        mainLayout = QVBoxLayout()
        self.setLayout(mainLayout)
        
        self.doupdate = False

    def slotRecord(self):
        self.doupdate = True
        for e, i in self.elements:
            if type(e) == QComboBox:
                if e.currentText() not in self.presets[i]:
                    self.presets[i].append(e.currentText())
        self.data = {i: self.typegetvalue(e) for e, i in self.elements}
        self.data.update({'valid': self.valid.isChecked()})
        self.accept()
    
    def slotValid(self, valid):
        for elem in self.elements:
            elem[0].setEnabled(not valid)
    
    def typegetvalue(self, elem):
        if type(elem) == QComboBox:
            return elem.currentText()
        if type(elem) == QTextEdit:
            return elem.toPlainText()
        if type(elem) in [QSpinBox, QDoubleSpinBox]:
            return elem.value()
        if type(elem) == QDateEdit:
            return elem.date().toString("yyyy-MM-dd")
        return elem.value()

    def addfield(self, klass, idstr, label, unit, default=None):
        from helpers import myfield
        from gridimage import GridImage
        from helpers import RadioWidget
        elem = klass(None)
        self.elements.append((elem, idstr))
        if type(elem) == QComboBox:
            elem.setEditable(True)
            if idstr in self.presets:
                elem.addItems(self.presets[idstr])
                elem.setCurrentIndex(elem.findText(self.data.get(idstr, default)))
        if type(elem) == QTextEdit:
            elem.setPlainText(self.data.get(idstr, default))
        if type(elem) in [QSpinBox, QDoubleSpinBox]:
            elem.setRange(-1000000, 1000000) # so that the value (next line) gets recorded
            if type(elem) == QDoubleSpinBox:
                elem.setDecimals(3) # so we don't get .9999999s while scolling
            elem.setValue(self.data.get(idstr, default))
        if type(elem) == QDateEdit:
            elem.setDate(QDate.fromString(self.data.get(idstr, default), "yyyy-MM-dd"))
        if type(elem) == GridImage:
            elem.load(self.data.get(idstr, default), self.location, self.imagesdir, self.gridname)
        if type(elem) == RadioWidget:
            elem.load(self.data.get(idstr, default))
        myfield(label, unit, elem, self.layout())
        
        return elem # so that it may be further tuned

    def addfieldbydict(self, fieldname):
        return self.addfield(self.fields[fieldname][0], fieldname, self.fields[fieldname][1], self.fields[fieldname][2], self.fields[fieldname][3])
        

    def addbottom(self):
        self.valid = QCheckBox("Validated")
        self.valid.toggled.connect(self.slotValid)
        self.layout().addWidget(self.valid)
        
        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.buttonBox.accepted.connect(self.slotRecord)
        self.buttonBox.rejected.connect(self.reject)
        self.layout().addWidget(self.buttonBox)
        
        self.valid.setChecked(self.data.get('valid', False))
