#  Author: Jonathan Michalon <michalon@igbmc.fr>, 2016
#  Institut de Génétique et de Biologie Moléculaire et Cellulaire
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from PyQt5.QtWidgets import *
from panebase import PaneBase

class NS(QGroupBox):

    def __init__(self, title, presets):
        super().__init__()
        
        layout = QHBoxLayout()        

        self.name = QComboBox(None)
        self.name.setEditable(True)
        self.name.addItems(presets['name'])
        layout.addWidget(self.name)
        
        self.stoechio = QDoubleSpinBox(None)
        layout.addWidget(self.stoechio)
        
        buttonBox = QDialogButtonBox(QDialogButtonBox.No)
        self.remove = buttonBox.button(QDialogButtonBox.No)
        self.remove.setText('Remove')
        layout.addWidget(buttonBox)
        
        self.setLayout(layout)
        self.setTitle(title)
        self.isFlat = True
        
    def value(self):
        return {self.title(): [self.name.currentText(), self.stoechio.value()]}
      
    def load(self, data):
        key, value = list(data.items())[0]
        self.setTitle(key)
        self.name.setCurrentIndex(self.name.findText(value[0]))
        self.stoechio.setValue(value[1])

class SampleDialog(PaneBase):
 
    fields = {
        'concentration': (QDoubleSpinBox, 'Concentration in Buffer', 'µg/ml', 0.2)
    }

    def __init__(self, parent, data={}, presets={}):
        if not presets:
            presets = {
                        'name': [],
                      }
        
        # must exist before loading data (in super's init)
        self.addButton = QPushButton("&Add Component")
        
        PaneBase.__init__(self, parent, "Sample Data", data, presets)
        
        from helpers import VariableWidget
        self.NSs = VariableWidget(NS, data.get('components', []), presets)
        self.elements.append((self.NSs, 'components'))
        self.layout().addWidget(self.NSs)
        
        e = self.addfieldbydict('concentration')
        e.setSingleStep(0.1)
        
        self.addbottom()

        self.buttonBox.addButton(self.addButton, QDialogButtonBox.HelpRole)
        self.addButton.clicked.connect(self.NSs.add)

    def slotRecord(self):
        for e in self.NSs.components:
            if e.name.currentText() not in self.presets['name']:
                self.presets['name'].append(e.name.currentText())
        super().slotRecord()

    def slotValid(self, valid):
        super().slotValid(valid)
        self.addButton.setEnabled(not valid)
